@@ -2,7 +2,7 @@
 # -*- coding: UTF-8 -*-
 
 # BleachBit
-# Copyright (C) 2008-2021 Andrew Ziem
+# Copyright (C) 2008-2023 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -50,8 +50,7 @@ def normalized_walk(top, **kwargs):
                 for fn in filenames
             ]
     else:
-        for result in walk(top, **kwargs):
-            yield result
+        yield from walk(top, **kwargs)
 
 
 Search = namedtuple('Search', ['command', 'regex', 'nregex', 'wholeregex', 'nwholeregex'])
@@ -103,11 +102,8 @@ class DeepScan:
         yield_time = time.time()
 
         for (top, searches) in self.searches.items():
-            compiled_searches = []
-            for s in searches:
-                compiled_searches.append(CompiledSearch(s))
-
-            for (dirpath, dirnames, filenames) in normalized_walk(top):
+            compiled_searches = [CompiledSearch(s) for s in searches]
+            for (dirpath, _dirnames, filenames) in normalized_walk(top):
                 for c in compiled_searches:
                     # fixme, don't match filename twice
                     for filename in filenames:
